/* Laser grid based camera trigger
 * Code for Arduino platform 
 *
 * Gaurav Manek, 2012
 * http://www.gauravmanek.com/blog/?p=58
 */

// Pin definitions
#define PIN_LDR 2
#define PIN_LASER 7
#define PIN_REMOTE 11
#define PIN_BEEPER 5

// Delay definitions 
#define DELAY_LASER 50
#define DELAY_CAMERA 250
#define DELAY_BEEP 500

int i = 0;

volatile boolean isTriggered = false;
boolean isEnabled = false;

void setup() {
  pinMode(PIN_REMOTE, OUTPUT);
  pinMode(PIN_LASER, OUTPUT);
  pinMode(PIN_BEEPER, OUTPUT);
  pinMode(PIN_LDR, INPUT);
  
  // Use interrupts to process the beam being broken.
  attachInterrupt(0, handleInterrupt, RISING);
  
  // Serial comms will allow us to enable/disable the laser
  Serial.begin(9600);
}

void handleInterrupt() {
  // Keep this function as short as possible.
  isTriggered = true;
}

void loop() {
  if(isTriggered && isEnabled) {
    Serial.print(i);
    Serial.write('\n');
    
    // Disable laser
    digitalWrite(PIN_LASER, LOW);
    
    // Trip the camera here
    digitalWrite(PIN_REMOTE, HIGH);
    delay(DELAY_CAMERA);
    digitalWrite(PIN_REMOTE, LOW);
    delay(DELAY_CAMERA);
    
    // Play beep
    digitalWrite(PIN_BEEPER, HIGH);
    delay(DELAY_BEEP);
    digitalWrite(PIN_BEEPER, LOW); 
    
    // Enable laser
    digitalWrite(PIN_LASER, HIGH);
    delay(DELAY_LASER);
    // Arm the system
    isTriggered = false;
  }
  while (Serial.available()){
    // Use serial to enable or disable the laser
    char tmp = Serial.read();
    switch(tmp) {
      // If lowercase 'e' (0x65) is recieved, enable it 
      case 'e':
        digitalWrite(PIN_LASER, HIGH);
        delay(DELAY_LASER);
        isEnabled = true;
        isTriggered = false;
        break;
      // If lowercase 'd' (0x64) is recieved, disable it
      case 'd':
        digitalWrite(PIN_LASER, LOW);
        isEnabled = false;
        break;
    }
  }
  i++;
}
